from django.utils.translation import gettext as _

from .base import RequestsMessengerBase
from ..exceptions import MessengerException


class VKontakteMessengerException(MessengerException):
    """Exceptions raised by VKontakte messenger."""


class VKontakteMessenger(RequestsMessengerBase):
    """Implements VKontakte page wall message publishing.

    Steps to be done:

    1. Create a user/community page.
    2. Create `Standalone` application at http://vk.com/apps?act=manage
    3. Get your Application ID (under Settings menu item in left menu)
    4. To generate an access token go to using your browser:

        https://oauth.vk.com/authorize?client_id={app_id}&scope=wall,offline&display=page&response_type=token
        &v=5.52&redirect_uri=https://oauth.vk.com/blank.html
        
        * Replace {app_id} with actual application ID.

    5. Copy token from URL in browser (symbols after `access_token=` but before &)
    6. Use this token.

    """
    alias = 'vk'
    title = _('VKontakte')

    address_attr = 'vkontakte'

    _url_wall = 'https://api.vk.com/method/wall.post'

    def __init__(self, access_token, proxy=None):
        """Configures messenger.

        :param str access_token: Unique authentication token to access your VK user/community page.
        :param dict|Callable: Dictionary of proxy settings,
            or a callable returning such a dictionary.

        """
        super().__init__(proxy=proxy)
        self.access_token = access_token

    def _send_message(self, msg, to=None):

        # Automatically deduce message type.
        message_type = 'attachments' if msg.startswith('http') else 'message'

        json = self.post(
            url=self._url_wall,
            data={
                message_type: msg,
                'owner_id': to,
                'from_group': 1,
                'access_token': self.access_token,
                'v': '5.73',
            })

        if 'error' in json:
            error = json['error']
            raise VKontakteMessengerException('%s: %s' % (error['error_code'], error['error_msg']))

        return json['response']['post_id']  # Returns post ID.



from traceback import format_exc

from django.core.management.base import BaseCommand

from sitemessage.compat import CommandOption, options_getter
from sitemessage.toolbox import check_undelivered


get_options = options_getter((
    CommandOption(
        '--to', action='store', dest='to', default=None,
        help='Recipient e-mail. If not set Django ADMINS setting is used.'
    ),
))


class Command(BaseCommand):

    help = 'Sends a notification email if any undelivered dispatches.'

    option_list = get_options()

    def add_arguments(self, parser):
        get_options(parser.add_argument)

    def handle(self, *args, **options):

        to = options.get('to', None)

        self.stdout.write('Checking for undelivered dispatches ...\n')

        try:
            undelivered_count = check_undelivered(to=to)

            self.stdout.write('Undelivered dispatches count: %s.\n' % undelivered_count)

        except Exception as e:
            self.stderr.write(self.style.ERROR('Error on check: %s\n%s' % (e, format_exc())))

        else:
            self.stdout.write('Check done.\n')



from django.utils.translation import gettext as _

from .base import RequestsMessengerBase
from ..exceptions import MessengerWarmupException, MessengerException


class TelegramMessengerException(MessengerException):
    """Exceptions raised by Telegram messenger."""


class TelegramMessenger(RequestsMessengerBase):
    """Implements Telegram message delivery via Telegram Bot API."""

    alias = 'telegram'
    title = _('Telegram')

    address_attr = 'telegram'

    _session_started = False
    _tpl_url = 'https://api.telegram.org/bot%(token)s/%(method)s'

    def __init__(self, auth_token, proxy=None):
        """Configures messenger.

        Register a Telegram Bot using instructions from https://core.telegram.org/bots/api

        :param auth_token: Bot unique authentication token
        """
        super().__init__(proxy=proxy)
        self.auth_token = auth_token

    def _verify_bot(self):
        """Sends an API command to test whether bot is authorized."""
        self._send_command('getMe')

    def get_updates(self):
        """Returns new messages addressed to bot.

        :rtype: list
        """
        with self.before_after_send_handling():
            result = self._send_command('getUpdates')
        return result

    def get_chat_ids(self):
        """Returns unique chat IDs from `/start` command messages sent to our bot by users.
        Those chat IDs can be used to send messages to chats.

        :rtype: list
        """
        updates = self.get_updates()
        chat_ids = []
        if updates:
            for update in updates:
                message = update['message']
                if message['text'] == '/start':
                    chat_ids.append(message['chat']['id'])
        return list(set(chat_ids))

    def before_send(self):
        if self._session_started:
            return

        try:
            self._verify_bot()
            self._session_started = True
        except TelegramMessengerException as e:
            raise MessengerWarmupException('Telegram Error: %s' % e)

    def _build_message(self, text, to=None):
        return {'chat_id': to, 'text': text}

    def _send_command(self, method_name, data=None):
        """Sends a command to API.

        :param str method_name:
        :param dict data:
        :return:
        """
        json = self.post(url=self._tpl_url % {'token': self.auth_token, 'method': method_name}, data=data)

        if not json['ok']:
            raise TelegramMessengerException(json['description'])

        return json['result']

    def _send_message(self, msg, to=None):
        return self._send_command('sendMessage', msg)

    def send(self, message_cls, message_model, dispatch_models):
        if not self._session_started:
            return
        super().send(message_cls, message_model, dispatch_models)



from traceback import format_exc

from django.core.management.base import BaseCommand

from sitemessage.compat import CommandOption, options_getter
from sitemessage.toolbox import cleanup_sent_messages


get_options = options_getter((
    CommandOption(
        '--ago', action='store', dest='ago', default=None, type=int,
        help='Allows cleanup messages sent X days ago. Defaults to None (cleanup all sent).'
    ),
    CommandOption('--dispatches_only', action='store_false', dest='dispatches_only', default=False,
                  help='Remove dispatches only (messages objects will stay intact).'),
))


class Command(BaseCommand):

    help = 'Removes sent dispatches from DB.'

    option_list = get_options()

    def add_arguments(self, parser):
        get_options(parser.add_argument)

    def handle(self, *args, **options):

        ago = options.get('ago', None)
        dispatches_only = options.get('dispatches_only', False)

        suffix = []

        if not dispatches_only:
            suffix.append('and messages')

        if ago:
            suffix.append('sent %s days ago' % ago)

        self.stdout.write('Cleaning up dispatches %s ...\n' % ' '.join(suffix))

        try:
            cleanup_sent_messages(ago=ago, dispatches_only=dispatches_only)

        except Exception as e:
            self.stderr.write(self.style.ERROR('Error on cleanup: %s\n%s' % (e, format_exc())))

        else:
            self.stdout.write('Cleanup done.\n')



from traceback import format_exc

from django.core.management.base import BaseCommand

from sitemessage.compat import CommandOption, options_getter
from sitemessage.toolbox import send_scheduled_messages


get_options = options_getter((
    CommandOption(
        '--priority', action='store', dest='priority', default=None,
        help='Allows to filter scheduled messages by a priority number. Defaults to None.'
    ),
))


class Command(BaseCommand):

    help = 'Sends scheduled messages (both in pending and error statuses).'

    option_list = get_options()

    def add_arguments(self, parser):
        get_options(parser.add_argument)

    def handle(self, *args, **options):
        priority = options.get('priority', None)
        priority_str = ''

        if priority is not None:
            priority_str = 'with priority %s ' % priority

        self.stdout.write('Sending scheduled messages %s ...\n' % priority_str)

        try:
            send_scheduled_messages(priority=priority)

        except Exception as e:
            self.stderr.write(self.style.ERROR('Error on send: %s\n%s' % (e, format_exc())))

        else:
            self.stdout.write('Sending done.\n')



from traceback import format_exc

from django.core.management.base import BaseCommand

from sitemessage.compat import CommandOption, options_getter
from sitemessage.toolbox import send_test_message


get_options = options_getter((
    CommandOption(
        '--to', action='store', dest='to', default=None,
        help='Recipient address (if supported by messenger).'
    ),
))


class Command(BaseCommand):

    help = 'Removes sent dispatches from DB.'

    option_list = get_options()
    args = '[messenger]'

    def add_arguments(self, parser):
        parser.add_argument('messenger', metavar='messenger', help='Messenger to test.')
        get_options(parser.add_argument)

    def handle(self, messenger, *args, **options):

        to = options.get('to', None)

        self.stdout.write('Sending test message using %s ...\n' % messenger)

        try:
            result = send_test_message(messenger, to=to)
            self.stdout.write('Probing function result: %s.\n' % result)

        except Exception as e:
            self.stderr.write(self.style.ERROR('Error on probe: %s\n%s' % (e, format_exc())))

        else:
            self.stdout.write('Probing done.\n')



from django.utils.translation import gettext as _

from .base import RequestsMessengerBase
from ..exceptions import MessengerException


class FacebookMessengerException(MessengerException):
    """Exceptions raised by Facebook messenger."""


class FacebookMessenger(RequestsMessengerBase):
    """Implements Facebook page wall message publishing.

    Steps to be done:

    1. Create FB application for your website at https://developers.facebook.com/apps/

    2. Create a page
       (possibly at https://developers.facebook.com/apps/{app_id}/settings/advanced/ under `App Page`
       - replace {app_id} with your application ID).

    3. Go to Graph API Explorer - https://developers.facebook.com/tools/explorer/
       3.1. Pick your application from top right dropdown.
       3.2. `Get User Token` using dropdown near Access Token field. Check `manage_pages` permission.
       
    4. Get page access token from your user token and application credentials using .get_page_access_token().

    """

    alias = 'fb'
    title = _('Facebook')

    _graph_version = '2.6'

    _url_base = 'https://graph.facebook.com'
    _url_versioned = _url_base + '/v' + _graph_version
    _tpl_url_feed = _url_versioned + '/%(page_id)s/feed'

    def __init__(self, page_access_token, proxy=None):
        """Configures messenger.

        :param str page_access_token: Unique authentication token of your FB page.
            One could be generated from User token using .get_page_access_token().

        """
        super().__init__(proxy=proxy)
        self.access_token = page_access_token

    def get_page_access_token(self, app_id, app_secret, user_token):
        """Returns a dictionary of never expired page token indexed by page names.

        :param str app_id: Application ID
        :param str app_secret: Application secret
        :param str user_token: User short-lived token
        :rtype: dict

        """
        url_extend = (
            self._url_base + '/oauth/access_token?grant_type=fb_exchange_token&'
                             'client_id=%(app_id)s&client_secret=%(app_secret)s&fb_exchange_token=%(user_token)s')

        response = self.get(url_extend % {'app_id': app_id, 'app_secret': app_secret, 'user_token': user_token})
        user_token_long_lived = response.split('=')[-1]

        json = self.get(self._url_versioned + '/me/accounts?access_token=%s' % user_token_long_lived, json=True)

        tokens = {item['name']: item['access_token'] for item in json['data'] if item.get('access_token')}

        return tokens

    def _send_message(self, msg, to=None):

        # Automatically deduce message type.
        message_type = 'link' if msg.startswith('http') else 'message'

        json = self.post(
            url=self._tpl_url_feed % {'page_id': 'me'},
            data={'access_token': self.access_token, message_type: msg})

        if 'error' in json:
            error = json['error']
            raise FacebookMessengerException('%s: %s' % (error['code'], error['message']))

        return json['id']  # Returns post ID.
